﻿#include <libxl.h>
#include <stdexcept>
#include "LoadFromExcel.h"
#include "../../excel/excel.h"

using namespace std;
using namespace libxl;


/************************************************
* \brief 从Excel文件读取车商方案表
* \param filePath Excel文件路径
* \param schemeMap 存放车商方案数据的map
************************************************/
void LoadCarDealerSchemeFromXlsx( const wstring &           filePath,
                                  unsigned int              sheetIndex,
                                  unsigned int              startRowIndex,
                                  vector<CarDealerScheme> & schemeVector )
{
	Book *  pBook  = xlCreateXMLBookW();
	Sheet * pSheet = nullptr;

	if ( pBook == nullptr )
	{
		throw runtime_error( "libxl库加载失败！" );
	}

	setKey( pBook );

	if ( pBook->load( filePath.c_str() ) != true )
	{
		string errorMessage = "打开文件失败！";
		errorMessage.append( pBook->errorMessage() );

		throw runtime_error( errorMessage );
	}

	pSheet = pBook->getSheet( sheetIndex );

	if ( pSheet == nullptr )
	{
		string errorMessage = "读取sheet失败！";
		errorMessage.append( pBook->errorMessage() );

		pBook->release();

		throw runtime_error( errorMessage );
	}

	int lastRowIndex     = pSheet->lastRow();
	int firstRowIndex    = pSheet->firstRow();
	int firstColumnIndex = pSheet->firstCol();
	int rowIndex         = firstRowIndex + startRowIndex;

	while ( rowIndex <= lastRowIndex )
	{
		int colunmIndex = firstRowIndex;

		const wstring && theYear       = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex, true );
		const wstring && theMonth      = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 1, true );
		const wstring && carDealerCode = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 2, true );
		const wstring && carDealerName = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 3, true );
		const wstring && manHourPrice  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 4, true );
		const wstring && partPrice     = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 5, true );
		const wstring && claimSupport  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 6, true );
		const wstring && scheme        = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 7, true );
		const wstring && isQualified   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 8, true );

		//空行跳过
		if ( carDealerCode.empty() == true )
		{
			rowIndex++;

			continue;
		}

		CarDealerScheme carDealerScheme( theYear,
		                                 theMonth,
		                                 carDealerCode,
		                                 manHourPrice,
		                                 partPrice,
		                                 claimSupport,
		                                 scheme,
		                                 isQualified );

		schemeVector.push_back( carDealerScheme );

		rowIndex++;
	}

	pBook->release();
}

void LoadCarDealerAchievementFromXlsx( const std::wstring &                filePath,
                                       unsigned                            sheetIndex,
                                       unsigned                            startRowIndex,
                                       std::vector<CarDealerAchievement> & achievementVector )
{
	Book *  pBook  = xlCreateXMLBookW();
	Sheet * pSheet = nullptr;

	if ( pBook == nullptr )
	{
		throw runtime_error( "libxl库加载失败！" );
	}

	setKey( pBook );

	if ( pBook->load( filePath.c_str() ) != true )
	{
		string errorMessage = "打开文件失败！";
		errorMessage.append( pBook->errorMessage() );

		throw runtime_error( errorMessage );
	}

	pSheet = pBook->getSheet( sheetIndex );

	if ( pSheet == nullptr )
	{
		string errorMessage = "读取sheet失败！";
		errorMessage.append( pBook->errorMessage() );

		pBook->release();

		throw runtime_error( errorMessage );
	}

	int lastRowIndex     = pSheet->lastRow();
	int firstRowIndex    = pSheet->firstRow();
	int firstColumnIndex = pSheet->firstCol();
	int rowIndex         = firstRowIndex + startRowIndex;

	while ( rowIndex <= lastRowIndex )
	{
		int colunmIndex = firstRowIndex;

		const wstring && theYear            = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex, true );
		const wstring && theMonth           = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 1, true );
		const wstring && carDealerCode      = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 2, true );
		const wstring && carDealerName      = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex + 3, true );
		long double      checkedAchievement = 0;
		int              policyAmount       = 0;
		int              cpicAmount         = 0;
		int              piccAmount         = 0;
		int              pinganAmount       = 0;
		int              othersAmount       = 0;

		//空行跳过
		if ( carDealerCode.empty() == true )
		{
			rowIndex++;

			continue;
		}

		//每个字段都要先判断数据类型再读写，防止填写表格的人填错内容。
		CellType type = pSheet->cellType( rowIndex, firstColumnIndex + 4 );
		char     errorMessage[1000];

		//产值
		if ( type == CELLTYPE_NUMBER )
		{
			checkedAchievement = pSheet->readNum( rowIndex, firstColumnIndex + 4 );
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 4 );

			throw runtime_error( errorMessage );
		}

		//签单数量
		type = pSheet->cellType( rowIndex, firstColumnIndex + 5 );

		if ( type == CELLTYPE_NUMBER )
		{
			policyAmount = static_cast<int>(pSheet->readNum( rowIndex, firstColumnIndex + 5 ));
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 5 );

			throw runtime_error( errorMessage );
		}

		//太平洋保险新车签单台次
		type = pSheet->cellType( rowIndex, firstColumnIndex + 6 );

		if ( type == CELLTYPE_NUMBER )
		{
			cpicAmount = static_cast<int>(pSheet->readNum( rowIndex, firstColumnIndex + 6 ));
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 6 );

			throw runtime_error( errorMessage );
		}

		//中国人保新车签单台次
		type = pSheet->cellType( rowIndex, firstColumnIndex + 7 );

		if ( type == CELLTYPE_NUMBER )
		{
			piccAmount = static_cast<int>(pSheet->readNum( rowIndex, firstColumnIndex + 7 ));
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 7 );

			throw runtime_error( errorMessage );
		}

		//中国平安新车签单台次
		type = pSheet->cellType( rowIndex, firstColumnIndex + 8 );

		if ( type == CELLTYPE_NUMBER )
		{
			pinganAmount = static_cast<int>(pSheet->readNum( rowIndex, firstColumnIndex + 8 ));
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 8 );

			throw runtime_error( errorMessage );
		}

		//其他保险公司新车签单台次
		type = pSheet->cellType( rowIndex, firstColumnIndex + 9 );

		if ( type == CELLTYPE_NUMBER )
		{
			othersAmount = static_cast<int>(pSheet->readNum( rowIndex, firstColumnIndex + 9 ));
		}
		else
		{
			pBook->release();

			sprintf( errorMessage, "第%d行第%d列，格式错误，不是数字类型！", rowIndex, firstColumnIndex + 9 );

			throw runtime_error( errorMessage );
		}

		CarDealerAchievement achievement( theYear,
		                                  theMonth,
		                                  carDealerCode,
		                                  checkedAchievement,
		                                  policyAmount,
		                                  cpicAmount,
		                                  piccAmount,
		                                  pinganAmount,
		                                  othersAmount );

		achievementVector.push_back( achievement );

		rowIndex++;
	}

	pBook->release();
}

void LoadRepairOrderFromXlsx( const std::wstring &       filePath,
                              unsigned                   sheetIndex,
                              unsigned                   startRowIndex,
                              std::vector<RepairOrder> & orderVector )
{
	Book *  pBook  = xlCreateXMLBookW();
	Sheet * pSheet = nullptr;

	if ( pBook == nullptr )
	{
		throw runtime_error( "libxl库加载失败！" );
	}

	setKey( pBook );

	if ( pBook->load( filePath.c_str() ) != true )
	{
		string errorMessage = "打开文件失败！";
		errorMessage.append( pBook->errorMessage() );

		throw runtime_error( errorMessage );
	}

	pSheet = pBook->getSheet( sheetIndex );

	if ( pSheet == nullptr )
	{
		string errorMessage = "读取sheet失败！";
		errorMessage.append( pBook->errorMessage() );

		pBook->release();

		throw runtime_error( errorMessage );
	}

	int lastRowIndex  = pSheet->lastRow();
	int firstRowIndex = pSheet->firstRow();
	int rowIndex      = firstRowIndex + startRowIndex;

	while ( rowIndex <= lastRowIndex )
	{
		int colunmIndex = pSheet->firstCol();

		const wstring && branchName                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && orderNo                     = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && orderType                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && notifyNo                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && damageArea                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && damageDate                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && generatingDate              = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && policyNo                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && policyNoJQX                 = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && plateNumber                 = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && brandName                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && isInsuranceObject           = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && isSuccess                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && recommandDealerCode         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && recommandDealerName         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && recommandDealerCodeInNotify = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && recommandDealerNameInNotify = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && recommandDealerNameInSurvey = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && agentName                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && surveyor                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && checkDate                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && repairingStartDate          = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && repairingFinishDate         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && status                      = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && lostItemID                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		const wstring && surveyorRecommandStatus     = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );

		//空行跳过
		if ( orderNo.empty() == true )
		{
			rowIndex++;

			continue;
		}

		RepairOrder order( branchName,
		                   orderNo,
		                   orderType,
		                   notifyNo,
		                   damageArea,
		                   damageDate,
		                   generatingDate,
		                   policyNo,
		                   policyNoJQX,
		                   plateNumber,
		                   brandName,
		                   isInsuranceObject,
		                   isSuccess,
		                   recommandDealerCode,
		                   recommandDealerName,
		                   recommandDealerCodeInNotify,
		                   recommandDealerNameInNotify,
		                   recommandDealerNameInSurvey,
		                   agentName,
		                   surveyor,
		                   checkDate,
		                   repairingStartDate,
		                   repairingFinishDate,
		                   status,
		                   lostItemID,
		                   surveyorRecommandStatus );

		orderVector.push_back( order );

		rowIndex++;
	}

	pBook->release();
}

void LoadRepairOrderFromXls( const std::wstring &       filePath,
                             unsigned                   sheetIndex,
                             unsigned                   startRowIndex,
                             std::vector<RepairOrder> & orderVector )
{
	Book *  pBook  = xlCreateBookW();
	Sheet * pSheet = nullptr;

	if ( pBook == nullptr )
	{
		throw runtime_error( "libxl库加载失败！" );
	}

	setKey( pBook );

	if ( pBook->load( filePath.c_str() ) != true )
	{
		string errorMessage = "打开文件失败！";
		errorMessage.append( pBook->errorMessage() );

		throw runtime_error( errorMessage );
	}

	pSheet = pBook->getSheet( sheetIndex );

	if ( pSheet == nullptr )
	{
		string errorMessage = "读取sheet失败！";
		errorMessage.append( pBook->errorMessage() );

		pBook->release();

		throw runtime_error( errorMessage );
	}

	int lastRowIndex  = pSheet->lastRow();
	int firstRowIndex = pSheet->firstRow();
	int rowIndex      = firstRowIndex + startRowIndex;

	while ( rowIndex <= lastRowIndex )
	{
		int colunmIndex = pSheet->firstCol();

		// const wstring && branchName                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && orderNo                     = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && orderType                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && notifyNo                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && damageArea                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && damageDate                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && generatingDate              = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && policyNo                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && policyNoJQX                 = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && plateNumber                 = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && brandName                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && isInsuranceObject           = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && isSuccess                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && recommandDealerCode         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && recommandDealerName         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && recommandDealerCodeInNotify = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && recommandDealerNameInNotify = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && recommandDealerNameInSurvey = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && agentName                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && surveyor                    = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && checkDate                   = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && repairingStartDate          = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && repairingFinishDate         = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && status                      = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && lostItemID                  = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		// const wstring && surveyorRecommandStatus     = ReadCellStringFromXlsx( pBook, sheetIndex, rowIndex, colunmIndex++, true );
		//
		// //空行跳过
		// if ( orderNo.empty() == true )
		// {
		// 	rowIndex++;
		//
		// 	continue;
		// }
		//
		// RepairOrder order( branchName,
		//                    orderNo,
		//                    orderType,
		//                    notifyNo,
		//                    damageArea,
		//                    damageDate,
		//                    generatingDate,
		//                    policyNo,
		//                    policyNoJQX,
		//                    plateNumber,
		//                    brandName,
		//                    isInsuranceObject,
		//                    isSuccess,
		//                    recommandDealerCode,
		//                    recommandDealerName,
		//                    recommandDealerCodeInNotify,
		//                    recommandDealerNameInNotify,
		//                    recommandDealerNameInSurvey,
		//                    agentName,
		//                    surveyor,
		//                    checkDate,
		//                    repairingStartDate,
		//                    repairingFinishDate,
		//                    status,
		//                    lostItemID,
		//                    surveyorRecommandStatus );
		//
		// orderVector.push_back( order );

		rowIndex++;
	}

	pBook->release();
}
