﻿#include <ocilib.hpp>
#include <stdexcept>
#include <QString>
#include "ImportToOracle.h"
#include "../../../db/ocilib/db_oper.h"

using namespace std;
using namespace ocilib;

void l_error_handler( OCI_Error * pError )
{
	string errorString = OCI_ErrorGetString( pError );

	std::runtime_error error( OCI_ErrorGetString( pError ) );

	throw error;
}

/*
void ImportCarDealerAchievementToOracle( std::string                         userName,
                                         std::string                         password,
                                         std::string                         tnsName,
                                         std::vector<CarDealerAchievement> & achievementVector )
{
	ostring sqlImport =
			"BEGIN "
			"  car_dealer.data_import_util_pkg.import_cardealer_achvmnt( :a_the_year, "
			"                                                            :a_the_month, "
			"                                                            :a_car_dealer_code, "
			"                                                            :a_checked_achievement, "
			"                                                            :a_policy_amount, "
			"                                                            :a_cpic_amount, "
			"                                                            :a_picc_amount , "
			"                                                            :a_pingan_amount, "
			"                                                            :a_others_amount ); "
			"END; ";

	//初始化
	try
	{
		Environment::Initialize();
	}
	catch ( exception & error )
	{
		string errorMessage = "ocilib初始化失败！";
		errorMessage.append( error.what() );

		throw runtime_error( errorMessage );
	}

	Connection * pConn = nullptr;
	Statement *  pStmt = nullptr;

	try
	{
		pConn = new Connection( userName, password, tnsName );
	}
	catch ( exception & error )
	{
		string errorMessage = "连接oracle失败！";
		errorMessage.append( error.what() );

		Environment::Cleanup();

		throw runtime_error( errorMessage );
	}

	try
	{
		pStmt = new Statement( *pConn );

		pStmt->Prepare( sqlImport );
		pStmt->AllowRebinding( true );

		for ( auto iter = achievementVector.begin();
		      iter != achievementVector.end();
		      ++iter )
		{
			string checkedAchievement = QString( "%1" ).arg( static_cast<double>(iter->getCheckedAchievement()) ).toStdString();
			string policyAmount       = QString( "%1" ).arg( iter->getPolicyAmount() ).toStdString();
			string cpicAmount         = QString( "%1" ).arg( iter->getCpicAmount() ).toStdString();
			string piccAmount         = QString( "%1" ).arg( iter->getPiccAmount() ).toStdString();
			string pinganAmount       = QString( "%1" ).arg( iter->getPinganAmount() ).toStdString();
			string othersAmount       = QString( "%1" ).arg( iter->getOthersAmount() ).toStdString();

			pStmt->Bind<ostring>( ostring(":a_the_year"), ostring(QString::fromStdWString( iter->getTheYear() ).toStdString().c_str()), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_the_month"), ostring(QString::fromStdWString( iter->getTheMonth() ).toStdString().c_str()), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_car_dealer_code"), ostring(QString::fromStdWString( iter->getCarDealerCode() ).toStdString().c_str()), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_checked_achievement"), ostring(checkedAchievement.c_str()), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_policy_amount"), ostring(policyAmount.c_str()), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_cpic_amount"), ostring(cpicAmount), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_picc_amount"), ostring(piccAmount), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_pingan_amount"), ostring(pinganAmount), BindInfo::BindDirectionValues::In );
			pStmt->Bind<ostring>(ostring(":a_others_amount"), ostring(othersAmount), BindInfo::BindDirectionValues::In );

			pStmt->ExecutePrepared();
		}
	}
	catch ( exception & error )
	{
		string errorMessage = "语句执行失败！";
		errorMessage.append( error.what() );

		Environment::Cleanup();

		throw runtime_error( errorMessage );
	}

	Environment::Cleanup();
}
*/

void ImportCarDealerAchievementToOracle( std::string                         userName,
                                         std::string                         password,
                                         std::string                         tnsName,
                                         std::vector<CarDealerAchievement> & achievementVector )
{
	OCI_Connection * pConnection = nullptr;
	OCI_Statement *  pStatement  = nullptr;
	int              returnCode  = 0;

	const otext * szSqlImport =
			"BEGIN "
			"  car_dealer.data_import_util_pkg.import_cardealer_achvmnt( :a_the_year, "
			"                                                            :a_the_month, "
			"                                                            :a_car_dealer_code, "
			"                                                            :a_checked_achievement, "
			"                                                            :a_policy_amount, "
			"                                                            :a_cpic_amount, "
			"                                                            :a_picc_amount , "
			"                                                            :a_pingan_amount, "
			"                                                            :a_others_amount ); "
			"END; ";

	//初始化
	returnCode = OCI_Initialize( l_error_handler, nullptr, OCI_ENV_DEFAULT );

	if ( static_cast<bool>(returnCode) == false )
	{
		string errorMessage( "ocilib初始化错误:" );
		errorMessage.append( get_last_error_message() );

		throw runtime_error( errorMessage );
	}

	pConnection = OCI_ConnectionCreate( tnsName.c_str(),
	                                    userName.c_str(),
	                                    password.c_str(),
	                                    OCI_SESSION_DEFAULT );

	pStatement = OCI_StatementCreate( pConnection );

	OCI_AllowRebinding( pStatement, true );
	OCI_Prepare( pStatement, szSqlImport );

	for ( auto iterAchievement = achievementVector.begin();
	      iterAchievement != achievementVector.end();
	      ++iterAchievement )
	{
		string checkedAchievement = QString( "%1" ).arg( static_cast<double>(iterAchievement->getCheckedAchievement()) ).toStdString();
		string policyAmount       = QString( "%1" ).arg( iterAchievement->getPolicyAmount() ).toStdString();
		string cpicAmount         = QString( "%1" ).arg( iterAchievement->getCpicAmount() ).toStdString();
		string piccAmount         = QString( "%1" ).arg( iterAchievement->getPiccAmount() ).toStdString();
		string pinganAmount       = QString( "%1" ).arg( iterAchievement->getPinganAmount() ).toStdString();
		string othersAmount       = QString( "%1" ).arg( iterAchievement->getOthersAmount() ).toStdString();
	}
}


void ImportRepairOrderToOracle( std::string                userName,
                                std::string                password,
                                std::string                tnsName,
                                std::vector<RepairOrder> & orderVector )
{
	const otext * szSqlImport =
			"BEGIN"
			"  car_dealer.dat:a_import_util_pkg.import_repairing_order(:a_branch_name,"
			"                                                          :a_order_no,"
			"                                                          :a_order_type,"
			"                                                          :a_notify_no,"
			"                                                          :a_damage_area,"
			"                                                          :a_damage_date,"
			"                                                          :a_generating_date,"
			"                                                          :a_policy_no,"
			"                                                          :a_policy_no_jqx,"
			"                                                          :a_plate_number,"
			"                                                          :a_brand_name,"
			"                                                          :a_is_insurance_object,"
			"                                                          :a_is_success,"
			"                                                          :a_recommend_dealer_code,"
			"                                                          :a_recommend_dealer_name,"
			"                                                          :a_recomm_dealer_code_in_notify,"
			"                                                          :a_recomm_dealer_name_in_notify,"
			"                                                          :a_recomm_dealer_name_in_survey,"
			"                                                          :a_agent_name,"
			"                                                          :a_surveyor,"
			"                                                          :a_check_date,"
			"                                                          :a_repairing_start_date,"
			"                                                          :a_repairing_finish_date,"
			"                                                          :a_status,"
			"                                                          :a_lost_item_id,"
			"                                                          :a_surveyor_recomm_status );"
			"END;";

	//初始化
}
